@@ -61,6 +61,7 @@ public:
         , xapianDatabase(nullptr)
         , xapianIndexExists(false)
         , compressEvents(false)
+        , frontendCaps(QApt::NoCaps)
     {
     }
     ~BackendPrivate()
@@ -116,6 +117,7 @@ public:
     bool writeSelectionFile(const QString &file, const QString &path) const;
     QString customProxy;
     QString initErrorMessage;
+    QApt::FrontendCaps frontendCaps;
 };
 
 bool BackendPrivate::writeSelectionFile(const QString &selectionDocument, const QString &path) const
@@ -574,7 +576,8 @@ PackageList Backend::search(const QString &searchString) const
       int top_percent = 0;
       for (Xapian::MSetIterator i = matches.begin(); i != matches.end(); ++i)
       {
-         Package* pkg = package(QLatin1String(i.get_document().get_data().c_str()));
+         std::string pkgName = i.get_document().get_data();
+         Package* pkg = package(QLatin1String(pkgName.c_str()));
          // Filter out results that apt doesn't know
          if (!pkg)
             continue;
@@ -651,6 +654,13 @@ bool Backend::isBroken() const
     return false;
 }
 
+QApt::FrontendCaps Backend::frontendCaps() const
+{
+    Q_D(const Backend);
+
+    return d->frontendCaps;
+}
+
 QDateTime Backend::timeCacheLastUpdated() const
 {
     QDateTime sinceUpdate;
@@ -1036,6 +1046,7 @@ QApt::Transaction * Backend::commitChanges()
 
     QDBusPendingReply<QString> rep = d->worker->commitChanges(packageList);
     Transaction *trans = new Transaction(rep.value());
+    trans->setFrontendCaps(d->frontendCaps);
 
     return trans;
 }
@@ -1053,6 +1064,7 @@ QApt::Transaction * Backend::installPackages(PackageList packages)
 
     QDBusPendingReply<QString> rep = d->worker->commitChanges(packageList);
     Transaction *trans = new Transaction(rep.value());
+    trans->setFrontendCaps(d->frontendCaps);
 
     return trans;
 }
@@ -1070,6 +1082,7 @@ QApt::Transaction * Backend::removePackages(PackageList packages)
 
     QDBusPendingReply<QString> rep = d->worker->commitChanges(packageList);
     Transaction *trans = new Transaction(rep.value());
+    trans->setFrontendCaps(d->frontendCaps);
 
     return trans;
 }
@@ -1106,6 +1119,7 @@ Transaction *Backend::downloadArchives(const QString &listFile, const QString &d
 
     QDBusPendingReply<QString> rep = d->worker->downloadArchives(packages, destination);
     Transaction *trans = new Transaction(rep.value());
+    trans->setFrontendCaps(d->frontendCaps);
 
     return trans;
 }
@@ -1116,6 +1130,7 @@ Transaction *Backend::installFile(const DebFile &debFile)
 
     QDBusPendingReply<QString> rep = d->worker->installFile(debFile.filePath());
     Transaction *trans = new Transaction(rep.value());
+    trans->setFrontendCaps(d->frontendCaps);
 
     return trans;
 }
@@ -1131,6 +1146,7 @@ Transaction *Backend::updateCache()
 
     QDBusPendingReply<QString> rep = d->worker->updateCache();
     Transaction *trans = new Transaction(rep.value());
+    trans->setFrontendCaps(d->frontendCaps);
 
     return trans;
 }
@@ -1142,6 +1158,7 @@ Transaction *Backend::upgradeSystem(UpgradeType upgradeType)
     bool safeUpgrade = (upgradeType == QApt::SafeUpgrade);
     QDBusPendingReply<QString> rep = d->worker->upgradeSystem(safeUpgrade);
     Transaction *trans = new Transaction(rep.value());
+    trans->setFrontendCaps(d->frontendCaps);
 
     return trans;
 }
@@ -1215,22 +1232,24 @@ bool Backend::loadSelections(const QString &path)
         line = line.trimmed();
 
         QByteArray aKey;
-        QByteArray aValue;
-        int eqpos = line.indexOf("\t\t");
+        std::string keyString;
+        const char *data = line.constData();
+        if (ParseQuoteWord(data, keyString) == false)
+            return false;
+        aKey = QByteArray(keyString.c_str());
 
-        if (eqpos < 0) {
-            // Invalid
-            lineIndex++;
-            continue;
-        } else {
-            aKey = line.left(eqpos);
-            aValue = line.right(line.size() - eqpos -2);
-        }
+        QByteArray aValue;
+        std::string valueString;
+        if (ParseQuoteWord(data, valueString) == false)
+            return false;
+        aValue = QByteArray(valueString.c_str());
 
         if (aValue.at(0) == 'i') {
             actionMap[aKey] = Package::ToInstall;
-        } else if ((aValue.at(0) == 'd') || (aValue.at(0) == 'u')) {
+        } else if ((aValue.at(0) == 'd') || (aValue.at(0) == 'u') || (aValue.at(0) == 'r')) {
             actionMap[aKey] = Package::ToRemove;
+        } else if (aValue.at(0) == 'p') {
+            actionMap[aKey] = Package::ToPurge;
         }
 
         ++lineIndex;
@@ -1265,6 +1284,10 @@ bool Backend::loadSelections(const QString &path)
                Fix.Remove(pkgIter);
                cache.MarkDelete(pkgIter, false);
                break;
+           case Package::ToPurge:
+               Fix.Remove(pkgIter);
+               cache.MarkDelete(pkgIter, true);
+               break;
         }
         ++mapIter;
     }
@@ -1354,6 +1377,7 @@ bool Backend::setPackagePinned(Package *package, bool pin)
 
             pkgTagFile tagFile(&Fd);
             if (_error->PendingError()) {
+                fclose(out);
                 return false;
             }
 
@@ -1375,6 +1399,7 @@ bool Backend::setPackagePinned(Package *package, bool pin)
                     fprintf(out, "\n");
                 }
             }
+            fclose(out);
 
             if (!tempFile.open()) {
                 return false;
@@ -1470,4 +1495,11 @@ bool Backend::addArchiveToCache(const DebFile &archive)
     return d->worker->copyArchiveToCache(archive.filePath());
 }
 
+void Backend::setFrontendCaps(FrontendCaps caps)
+{
+    Q_D(Backend);
+
+    d->frontendCaps = caps;
+}
+
 }
